var char_vector;
var stroke_info;
var canvas, context;
var default_stroke_color = '#cccccc';
var animated_stroke_color = '#000000';
var animating_stroke_color = '#339900';
var radical_color = '#00ff00';
var animate_speed_step = 40;
var animate_speed_time = 10;
var animate_interval_time = 500;
var reanimate_interval_time = 1000;
var animate_timers = [];
function show_stroke_order(jsonData, obj, isloop) {
    if(jsonData){
    	if(typeof isloop =='undefined') isloop = false;
        init(jsonData.cvec, jsonData.sorder, obj);
        clear_timers();
        clear_canvas();
        draw_default();
        stroke_order_animate(isloop);
    }
}

function Point (x, y, t) {this.x = x;this.y = y;this.t = t;}

function init(cvec, sorder,obj) {
    canvas = document.getElementById(obj);
    context = canvas.getContext("2d");
    parse_char_vector(cvec);
    parse_stroke_info(sorder);
}

function clear_canvas(){
    context.clearRect(0, 0, canvas.width, canvas.height);
}

function clear_timers(){
    for (time in animate_timers){
        clearTimeout(time);
    }
    animate_timers = [];
}

function draw_default() {
    context.save();
    context.beginPath();
    create_char_path();
    context.fillStyle = default_stroke_color;
    context.fill();
    context.restore();
}

function show_radical(cvec, sorder){
    init(cvec, sorder);
    context.save();
    for (var i = 0; i < char_vector.length; i++){
        context.beginPath();
        create_stroke_path(i);
        context.fillStyle = is_radical_stroke(i) ? radical_color : default_stroke_color;
        context.fill();
    }
    context.restore();
}

function parse_char_vector (cvec){
    char_vector = [];
    
    for (var i = 2; i < cvec.length;){
        var numberOfOneStroke = parseInt(cvec.substr(i, 3), 10);
        i = i + 3;
        var stroke_points = [];
        for (j = 0; j < numberOfOneStroke; j++){
            var x = parseInt(cvec.substr(i, 4), 16) ^ 62953;
            i += 4;
            var y = parseInt(cvec.substr(i, 4), 16) ^ 10935;
            i += 4;
            var t = cvec.charAt(i);
            i++;
            stroke_points.push(new Point(x, y, t == "1"));
        }
        char_vector.push(stroke_points);
    }
}

function parse_stroke_info(sorder){
    stroke_info = [];
    for (i = 0; i < sorder.length; i += 2){
        stroke_info.push(parseInt(sorder.substr(i, 2), 16));
    }
}

function create_char_path (){
    for (var i = 0; i < char_vector.length; i++){
        create_stroke_path(i);
    }
}

function create_stroke_path(index){
    var points = char_vector[index];
    var startPoint = points[0];
    context.moveTo(startPoint.x, startPoint.y);
    var prevPoint = startPoint;
    
    for (j = 1; j <= points.length; j++){
        var currentPoint = points[j%points.length];
        if (currentPoint.t){
            context.lineTo(currentPoint.x, currentPoint.y);
            prevPoint = currentPoint;
        }else{
            var nextPoint = points[(j+1)%points.length];
            context.quadraticCurveTo(currentPoint.x, currentPoint.y, nextPoint.x, nextPoint.y);
            prevPoint = nextPoint;
            j++;
        }
    }
}

function stroke_order_animate(loop){
    var animating_index = 0;
    var show_animate = function(){
        var direction = stroke_info[animating_index] & 3;
        if (animating_index <  char_vector.length){
            animate_fill(animating_index, direction, this.callback);
        }else{
            if (loop){
                animate_timers.push(setTimeout(function(){
                                               clear_timers();
                                               clear_canvas();
                                               draw_default();
                                               stroke_order_animate(loop);
                                               }, reanimate_interval_time));
            } else { // 如果不是循环播放
                //window.location = "native://word_stroke_complete";
            }
        }
    }

    this.callback = function(){
        if (continual_stroke_num(animating_index++) > 0){
            show_animate();
        }else{
            animate_timers.push(setTimeout(show_animate, animate_interval_time));
        }
    }
    
    show_animate();
}

function stroke_min_rect(index){
    var minX = 10000, minY = 10000, maxX = 0, maxY = 0;
    var points = char_vector[index];
    for (var i = 0; i < points.length; i++){
        var point = points[i];
        (minX > point.x) && (minX = point.x);
        (minY > point.y) && (minY = point.y);
        (maxX < point.x) && (maxX = point.x);
        (maxY < point.y) && (maxY = point.y);
    }
    return [minX, minY, maxX, maxY];
}

function animate_fill(index, direction, callback){
    var rect = stroke_min_rect(index);
    var x = rect[0], y = rect[1], w = rect[2] - rect[0], h = rect[3] - rect[1];
    var filled = animate_speed_step;
    var need_fill = (direction == 0 || direction == 2) ? w : h;
    fill_rect = function(rx, ry, rw, rh){
        context.save();
        context.beginPath();
        context.rect(rx, ry, rw, rh);
        context.clip();
        context.beginPath();
        create_stroke_path(index);
        context.fillStyle = animating_stroke_color;
        context.fill();
        context.restore();
    }

    draw_animated_stroke = function(){
        context.save();
        context.beginPath();
        
        for (var i = 0; i <= index; i++){
            create_stroke_path(i);
        }
        
        context.fillStyle = animated_stroke_color;
        context.fill();
        context.restore();
        
        callback();
    }
    
    draw = function(){
        if (direction == 0) { fill_rect(x, y, filled, h);}
        else if (direction == 1) { fill_rect(x, y, w, filled); }
        else if (direction == 2) { fill_rect(x+w-filled, y, filled, h); }
        else if (direction == 3) { fill_rect(x, y+h-filled, w, filled); }
        if (filled < need_fill){
            filled += animate_speed_step;
            animate_timers.push(setTimeout(draw, animate_speed_time));
        }else{
            if (continual_stroke_num(index) > 0){
                callback();
            }else{
                animate_timers.push(setTimeout(function(){draw_animated_stroke()}, animate_interval_time));
            }
        }
    }

    draw();
}

function is_continual(index){
    return (stroke_info[index] & 128) > 0;
}

function continual_stroke_num(index){
    var n = 0;
    
    for (var i = index + 1; i < stroke_info.length; i++){
        if (!is_continual(i)){
            break;
        }
        
        n++;
    }
    
    return n;
}

function stroke_start_index (index){
    for (var i = index; i >= 0; i--){
        if (!is_continual(i)){
            return i;
        }
    }
    
    return 0;
}

function is_radical_stroke(index){
    if ((stroke_info[index] & 64) > 0){
        return true;
    }else{
        return (stroke_info[stroke_start_index(index)] & 64) > 0;
    }
}

function stroke_number(){
    var n = 0;
    
    for (var i = 0; i < stroke_info.length; i++){
        if (!is_continual(i)){
            n++;
        }
    }
    
    return n;
}
